@@ -3,7 +3,6 @@ import copy
 import pybats
 import numpy as np
 import pandas as pd
-from typing import List
 from scipy import stats
 from pybats_detection.smooth import Smoothing
 from pybats_detection.utils import tidy_parameters
@@ -16,8 +15,7 @@ class Monitoring:
     of class `pybats.dglm.dlm`.
     """
 
-    def __init__(self, mod: pybats.dglm.dlm, prior_length: int = 10,
-                 bilateral: bool = False, smooth: bool = True,
+    def __init__(self, mod: pybats.dglm.dlm, smooth: bool = True,
                  interval: bool = True, level: float = 0.05):
         """Automatic Monitoring Analysis.
 
@@ -28,12 +26,6 @@ class Monitoring:
         ----------
         mod : pybats.dglm.dlm
             An object of class `pybats.dglm.dlm` with the defined DLM.
-        prior_length : int
-            Number of observation with the monitor off.
-        bilateral : bool
-            Should bilateral monitoring be performed?
-        return_state_parameters : bool
-            Should state parameters be returned?
         smooth : bool
             Should compute the smoothing moments?
         interval : bool
@@ -46,15 +38,8 @@ class Monitoring:
         ----------
         _mod : pybats.dglm.dlm
             An object of class `pybats.dglm.dlm` with the DLM updated.
-        _pd_y : pd.Series
-            Observed values of time series.
-        _prior_length : int
-            Number of observation with the monitor off.
-        _bilateral : bool
-            Indicate if the bilateral monitoring was performed.
-        _return_state_parameters : bool
-            Indicate if the a dictionary with the state parameters was
-            returned.
+        _smooth : bool
+            Indicate if the smooth distribution was calculated.
         _interval : bool
             Indicate if the credible interval was calculated.
         _level : double
@@ -62,44 +47,51 @@ class Monitoring:
 
         """
         self._mod = mod
-        self._prior_length = prior_length
-        self._bilateral = bilateral
         self._smooth = smooth
         self._interval = interval
         self._level = level
 
-    def fit(self, y: pd.Series, h: int = 4, tau: float = 0.135,
-            change_var: List = [5], distr: str = "normal",
-            type: str = "location", verbose: bool = True):
+    def fit(self, y: pd.Series, X: pd.DataFrame = None, prior_length: int = 10,
+            bilateral: bool = False, h: int = 4, tau: float = 0.135,
+            discount_factors: dict = {
+                "trend": 0.10, "seasonal": 0.90, "reg": 0.98},
+            distr_fam: str = "normal",
+            distr_type: str = "location", verbose: bool = True):
         """Perform the fit with automatic monitoring.
 
         Filtering and smoothing distribution with automatic monitoring on
         objects of class `pybats.dglm.dlm`.
 
-
         Parameters
         ----------
         y : pd.Series
             Observed values of time series.
+        X : pd.DataFrame
+            Observed values of fixed covariates with dimension `p` by `n`,
+            where `p` is the number of covariates.
+        prior_length : int
+            Number of observation with the monitor off.
+        bilateral : bool
+            Should bilateral monitoring be performed?
         h : double
             Value of change in the scale or location of the predictive
-            distrbution.
+            distribution.
         tau : double
             Threshold to compare the Bayes factor.
-        change_var : List
-            Values to increase the uncertainty on state space parameter by
-            multiplying prior covariance matrix, R, when the monitor detects
-            potential outlier or parametric change.
+        discount_factors : dict
+            Discount factors values to increase the uncertainty about
+            state space parameter, when the monitor detects potential outlier
+            or parametric change.
 
-            If length of `change_var` is different to number of parameters use,
-            then multiply the covariance matrix by the first value in the List.
-
-            The covariance terms are multiplied by the minimum value in
-            `change_var`.
-        distr : str
+            The dictionary should contain values with the following keys
+            representing the model blocks:
+                - `trend`: level and growth
+                - `seasonal`: seasonality
+                - `reg`: regressors
+        distr_fam : str
             Bayes factor distribution family. It could be "normal" or
             "tstudent".
-        type : str
+        distr_type : str
             Alternative distribution use to compute the Bayes factor.
             It could be "location" to detects change in the location of the
             distribution or "scale" to dectecs changes in the scale/dispersion
@@ -134,41 +126,52 @@ class Monitoring:
         Association 81, 741–750.
 
         """
+        n = len(y)
+        # Create a None vector when there are no regression covariates
+        if X is None:
+            X = pd.DataFrame(np.array([None]*(n+1)).reshape(-1, 1))
+
         self._pd_y = y.copy()
+        self._pd_X = X.copy()
         self._verbose = verbose
+        self._prior_length = prior_length
 
-        # Constructing a matrix for multiply the prior covariance matrix R
-        n_parms = len(self._mod.get_coef())
-
-        if len(change_var) != n_parms:
-            c_mat = np.full(shape=(n_parms, n_parms), fill_value=change_var[0])
-
-        if len(change_var) == n_parms:
-            c_mat = np.eye(n_parms)
-            np.fill_diagonal(c_mat, val=np.array(change_var))
-            # Change in covariance terms
-            c_mat[c_mat == 0] = np.min(np.array(change_var))
+        # Check discount factor parameters
+        discount_factors["level"] = (
+            self._mod.deltrend if discount_factors.get("level") is None
+            else discount_factors["level"])
+        discount_factors["seasonal"] = (
+            self._mod.delseas if discount_factors.get("seasonal") is None
+            else discount_factors["seasonal"])
+        discount_factors["reg"] = (
+            self._mod.delregn if discount_factors.get("reg") is None
+            else discount_factors["reg"])
+        self._discount_intervention = discount_factors
 
         if type == "scale":
-            return self._fit_unilateral(h=h, tau=tau, c_mat=c_mat,
-                                        distr=distr, type=type)
+            return self._fit_unilateral(h=h, tau=tau,
+                                        distr_fam=distr_fam,
+                                        distr_type=distr_type)
 
-        if self._bilateral:
-            return self._fit_bilateral(h=h, tau=tau, c_mat=c_mat,
-                                       distr=distr, type=type)
+        if bilateral:
+            return self._fit_bilateral(h=h, tau=tau,
+                                       distr_fam=distr_fam,
+                                       distr_type=distr_type)
         else:
-            return self._fit_unilateral(h=h, tau=tau, c_mat=c_mat,
-                                        distr=distr, type=type)
+            return self._fit_unilateral(h=h, tau=tau,
+                                        distr_fam=distr_fam,
+                                        distr_type=distr_type)
 
-    def _fit_unilateral(self, h, tau, c_mat, distr, type):
+    def _fit_unilateral(self, h, tau, distr_fam, distr_type):
         mod = copy.deepcopy(self._mod)
-        pd_y = self._pd_y
+        pd_y = self._pd_y.copy()
+        pd_X = self._pd_X.copy()
         n = len(pd_y)
 
         # Dictionaries to keep state and predictive parameters
         dict_state_parms = {
             "prior": {"a": [], "R": []},
-            "posterior": {"m": [], "C": [], "s": [], "df": []}
+            "posterior": {"m": [], "C": [], "df": [], "s": []}
         }
         dict_predictive = {
             "t": [], "prior": [], "y": [], "f": [], "q": [], "e": [], "df": [],
@@ -176,7 +179,7 @@ class Monitoring:
         lt_model_history = []
 
         # Get the Bayes Factor function
-        bf = self._bayes_factor(distr=distr, type=type)
+        bf = self._bayes_factor(distr_fam=distr_fam, distr_type=distr_type)
 
         # Fitting the model with the monitor off
         for t in range(0, self._prior_length):
@@ -184,10 +187,11 @@ class Monitoring:
             lt_model_history.append(copy.deepcopy(mod))
 
             # The observation at time t
-            yt = pd_y[t].copy()
+            yt = pd_y.values[t]
+            Xt = pd_X.values[t, :]
 
             # Get mean and variance one step ahead of forecast distribution
-            ft, qt = mod.forecast_marginal(k=1, state_mean_var=True)
+            ft, qt = mod.forecast_marginal(k=1, X=Xt, state_mean_var=True)
             ft = ft[0][0]
             qt = qt[0][0]
             et = (yt - ft) / np.sqrt(qt)
@@ -195,14 +199,15 @@ class Monitoring:
             # Saving prior state parameters
             dict_state_parms["prior"]["a"].append(mod.a)
             dict_state_parms["prior"]["R"].append(mod.R)
-            dict_state_parms["posterior"]["s"].append(mod.s)
 
             # Update model
-            mod.update(y=yt)
+            mod.update(y=yt, X=Xt)
 
             # Saving posterior state parameters
             dict_state_parms["posterior"]["m"].append(mod.m)
             dict_state_parms["posterior"]["C"].append(mod.C)
+            dict_state_parms["posterior"]["s"].append(mod.s[0][0])
+            dict_state_parms["posterior"]["df"].append(mod.n-1)
 
             # Saving predictive parameters
             dict_predictive["prior"].append(True)
@@ -223,10 +228,11 @@ class Monitoring:
             lt_model_history.append(copy.deepcopy(mod))
 
             # The observation at time t
-            yt = pd_y.values[t].copy()
+            yt = pd_y.values[t]
+            Xt = pd_X.values[t, :]
 
             # Get mean and variance one step ahead of forecast distribution
-            ft, qt = mod.forecast_marginal(k=1, state_mean_var=True)
+            ft, qt = mod.forecast_marginal(k=1, X=Xt, state_mean_var=True)
             ft = ft[0][0]
             qt = qt[0][0]
 
@@ -238,14 +244,18 @@ class Monitoring:
 
             # Cumulative Bayes factor (Lt)
             Lt = 1
-            if t > 0:
-                Lt = Ht * np.min([1, dict_predictive["L"][t - 1]])
+            if np.isnan(Ht):
+                Lt = dict_predictive["L"][t - 1]
+                lt = dict_predictive["l"][t - 1]
+            else:
+                if t > 0:
+                    Lt = Ht * np.min([1, dict_predictive["L"][t - 1]])
 
-            # Run length (lt)
-            lt = 1
-            if t > 0:
-                lt = np.where(dict_predictive["L"][t - 1] < 1,
-                              dict_predictive["l"][t - 1] + 1, 1)
+                # Run length (lt)
+                lt = 1
+                if t > 0:
+                    lt = np.where(dict_predictive["L"][t - 1] < 1,
+                                  dict_predictive["l"][t - 1] + 1, 1)
 
             dict_predictive["what_detected"].append("nothing")
             # Check structural change in parameters
@@ -258,16 +268,19 @@ class Monitoring:
                 # Change prior variance and get back in time (t - lt + 1)
                 index = t - lt + 1
                 mod = copy.deepcopy(lt_model_history[index])
-                mod.R = c_mat * mod.R
+                self._increase_uncertainty(
+                    model=mod, discount_factors=self._discount_intervention)
                 for i in range(index, t):
-                    ft, qt = mod.forecast_marginal(k=1, state_mean_var=True)
+                    ft, qt = mod.forecast_marginal(k=1, X=pd_X.values[i, :],
+                                                   state_mean_var=True)
                     dict_predictive["f"][i] = ft[0][0]
                     dict_predictive["q"][i] = qt[0][0]
-                    mod.update(y=pd_y.values[i])
+                    mod.update(y=pd_y.values[i], X=pd_X.values[i, :])
+
                 Lt = 1
                 lt = 0
                 # Compute new adjust forecast mean and variance
-                ft, qt = mod.forecast_marginal(k=1, state_mean_var=True)
+                ft, qt = mod.forecast_marginal(k=1, X=Xt, state_mean_var=True)
                 ft = ft[0][0]
                 qt = qt[0][0]
 
@@ -287,17 +300,20 @@ class Monitoring:
             # Saving prior state parameters
             dict_state_parms["prior"]["a"].append(mod.a)
             dict_state_parms["prior"]["R"].append(mod.R)
-            dict_state_parms["posterior"]["s"].append(mod.s[0][0])
 
             # Update model
-            mod.update(y=yt)
+            mod.update(y=yt, X=Xt)
 
+            # Increase uncertainty
             if potential_outlier:
-                mod.R = c_mat * mod.R
+                self._increase_uncertainty(
+                    model=mod, discount_factors=self._discount_intervention)
 
             # Saving posterior state parameters
             dict_state_parms["posterior"]["m"].append(mod.m)
             dict_state_parms["posterior"]["C"].append(mod.C)
+            dict_state_parms["posterior"]["s"].append(mod.s[0][0])
+            dict_state_parms["posterior"]["df"].append(mod.n-1)
 
             # Saving predictive parameters
             dict_predictive["t"].append(t+1)
@@ -312,6 +328,9 @@ class Monitoring:
             dict_predictive["l"].append(lt)
             # end loop
 
+        df_predictive = pd.DataFrame(dict_predictive)
+
+        # Organize the posterior parameters
         df_posterior = tidy_parameters(
             dict_parameters=dict_state_parms["posterior"],
             entry_m="m", entry_v="C",
@@ -322,10 +341,16 @@ class Monitoring:
         t_index = np.arange(0, len(df_posterior) / n_parms) + 1
         df_posterior["t"] = np.repeat(t_index, n_parms)
         df_posterior["t"] = df_posterior["t"].astype(int)
-        df_posterior = df_posterior[
-            ["t", "parameter", "mean", "variance"]].copy()
+        df_posterior = df_posterior.merge(
+            df_predictive[['t', 'df']], on="t", how="left",
+            validate="many_to_one")
 
-        df_predictive = pd.DataFrame(dict_predictive)
+        df_posterior = df_posterior[
+            ["t", "parameter", "mean", "variance", "df"]].copy()
+        df_posterior = df_posterior.sort_values(
+            ["parameter", "t"]).reset_index(drop=True)
+
+        # Compute credible intervals
         if self._interval:
             df_predictive["ci_lower"] = stats.t.ppf(
                 q=self._level/2, df=df_predictive["df"].values,
@@ -337,36 +362,43 @@ class Monitoring:
                 scale=np.sqrt(df_predictive["q"].values))
 
             df_posterior["ci_lower"] = stats.t.ppf(
-                q=self._level/2, df=df_posterior["t"].values + 1,
+                q=self._level/2, df=df_posterior["df"].values,
                 loc=df_posterior["mean"].values,
                 scale=np.sqrt(df_posterior["variance"].values))
             df_posterior["ci_upper"] = stats.t.ppf(
                 q=1-self._level/2,
-                df=df_posterior["t"].values + 1,
+                df=df_posterior["df"].values,
                 loc=df_posterior["mean"].values,
                 scale=np.sqrt(df_posterior["variance"].values))
 
+        # Output
         out = {"predictive": df_predictive, "posterior": df_posterior}
+
+        # Compute smooth distribution
         if self._smooth:
             smooth_class = Smoothing(
                 mod=self._mod, interval=self._interval, level=self._level)
             dict_smooth = smooth_class.fit(
-                y=pd_y, dict_state_parms=dict_state_parms)
+                y=pd_y, X=pd_X, dict_state_parms=dict_state_parms)
             out = {"filter": out, "smooth": dict_smooth}
 
+        mod.deltrend = self._mod.deltrend
+        mod.delregn = self._mod.delregn
+        mod.delseas = self._mod.delseas
         out["model"] = mod
 
         return out
 
-    def _fit_bilateral(self, h, tau, c_mat, distr, type):
+    def _fit_bilateral(self, h, tau, distr_fam, distr_type):
         mod = copy.deepcopy(self._mod)
-        pd_y = self._pd_y
+        pd_y = self._pd_y.copy()
+        pd_X = self._pd_X.copy()
         n = len(pd_y)
 
         # Dictionaries to keep state and predictive parameters
         dict_state_parms = {
             "prior": {"a": [], "R": []},
-            "posterior": {"m": [], "C": [], "s": [], "df": []}
+            "posterior": {"m": [], "C": [], "df": [], "s": []}
         }
         dict_predictive = {
             "t": [], "prior": [], "y": [], "f": [], "q": [], "e": [], "df": [],
@@ -376,7 +408,7 @@ class Monitoring:
         lt_model_history = []
 
         # Get the Bayes Factor function
-        bf = self._bayes_factor(distr=distr, type=type)
+        bf = self._bayes_factor(distr_fam=distr_fam, distr_type=distr_type)
 
         # Fitting the model with the monitor off
         for t in range(0, self._prior_length):
@@ -384,10 +416,11 @@ class Monitoring:
             lt_model_history.append(copy.deepcopy(mod))
 
             # The observation at time t
-            yt = pd_y[t].copy()
+            yt = pd_y.values[t]
+            Xt = pd_X.values[t, :]
 
             # Get mean and variance one step ahead of forecast distribution
-            ft, qt = mod.forecast_marginal(k=1, state_mean_var=True)
+            ft, qt = mod.forecast_marginal(k=1, X=Xt, state_mean_var=True)
             ft = ft[0][0]
             qt = qt[0][0]
             et = (yt - ft) / np.sqrt(qt)
@@ -395,14 +428,15 @@ class Monitoring:
             # Saving prior state parameters
             dict_state_parms["prior"]["a"].append(mod.a)
             dict_state_parms["prior"]["R"].append(mod.R)
-            dict_state_parms["posterior"]["s"].append(mod.s)
 
             # Update model
-            mod.update(y=yt)
+            mod.update(y=yt, X=Xt)
 
             # Saving posterior state parameters
             dict_state_parms["posterior"]["m"].append(mod.m)
             dict_state_parms["posterior"]["C"].append(mod.C)
+            dict_state_parms["posterior"]["s"].append(mod.s[0][0])
+            dict_state_parms["posterior"]["df"].append(mod.n-1)
 
             # Saving predictive parameters
             dict_predictive["t"].append(t+1)
@@ -425,10 +459,11 @@ class Monitoring:
             lt_model_history.append(copy.deepcopy(mod))
 
             # The observation at time t
-            yt = pd_y.values[t].copy()
+            yt = pd_y.values[t]
+            Xt = pd_X.values[t, :]
 
             # Get mean and variance one step ahead of forecast distribution
-            ft, qt = mod.forecast_marginal(k=1, state_mean_var=True)
+            ft, qt = mod.forecast_marginal(k=1, X=Xt, state_mean_var=True)
             ft = ft[0][0]
             qt = qt[0][0]
 
@@ -441,17 +476,26 @@ class Monitoring:
 
             # Cumulative Bayes factor (Lt)
             Lt = [1, 1]
-            if t > 0:
-                Lt[0] = Ht[0] * np.min([1, dict_predictive["L_upper"][t - 1]])
-                Lt[1] = Ht[1] * np.min([1, dict_predictive["L_lower"][t - 1]])
-
-            # Run length (lt)
             lt = [1, 1]
-            if t > 0:
-                lt[0] = np.where(dict_predictive["L_upper"][t - 1] < 1,
-                                 dict_predictive["l_upper"][t - 1] + 1, 1)
-                lt[1] = np.where(dict_predictive["L_lower"][t - 1] < 1,
-                                 dict_predictive["l_lower"][t - 1] + 1, 1)
+            if np.isnan(Ht[0]):
+                Lt[0] = dict_predictive["L_upper"][t - 1]
+                Lt[1] = dict_predictive["L_lower"][t - 1]
+
+                lt[0] = dict_predictive["l_upper"][t - 1]
+                lt[1] = dict_predictive["l_lower"][t - 1]
+            else:
+                if t > 0:
+                    Lt[0] = Ht[0] * np.min(
+                        [1, dict_predictive["L_upper"][t - 1]])
+                    Lt[1] = Ht[1] * np.min(
+                        [1, dict_predictive["L_lower"][t - 1]])
+
+                # Run length (lt)
+                if t > 0:
+                    lt[0] = np.where(dict_predictive["L_upper"][t - 1] < 1,
+                                     dict_predictive["l_upper"][t - 1] + 1, 1)
+                    lt[1] = np.where(dict_predictive["L_lower"][t - 1] < 1,
+                                     dict_predictive["l_lower"][t - 1] + 1, 1)
 
             min_Ht = min(Ht)
             min_Lt = min(Lt)
@@ -471,16 +515,18 @@ class Monitoring:
                 # Change prior variance and get back in time (t - lt + 1)
                 index = t - max_lt + 1
                 mod = copy.deepcopy(lt_model_history[index])
-                mod.R = c_mat * mod.R
+                self._increase_uncertainty(
+                    model=mod, discount_factors=self._discount_intervention)
                 for i in range(index, t):
-                    ft, qt = mod.forecast_marginal(k=1, state_mean_var=True)
+                    ft, qt = mod.forecast_marginal(k=1, X=pd_X.values[i, :],
+                                                   state_mean_var=True)
                     dict_predictive["f"][i] = ft[0][0]
                     dict_predictive["q"][i] = qt[0][0]
-                    mod.update(y=pd_y.values[i])
+                    mod.update(y=pd_y.values[i], X=pd_X.values[i, :])
                 Lt[arg] = 1
                 lt[arg] = 0
                 # Compute new adjust forecast mean and variance
-                ft, qt = mod.forecast_marginal(k=1, state_mean_var=True)
+                ft, qt = mod.forecast_marginal(k=1, X=Xt, state_mean_var=True)
                 ft = ft[0][0]
                 qt = qt[0][0]
 
@@ -502,17 +548,19 @@ class Monitoring:
             # Saving prior state parameters
             dict_state_parms["prior"]["a"].append(mod.a)
             dict_state_parms["prior"]["R"].append(mod.R)
-            dict_state_parms["posterior"]["s"].append(mod.s[0][0])
 
             # Update model
-            mod.update(y=yt)
+            mod.update(y=yt, X=Xt)
 
             if potential_outlier:
-                mod.R = c_mat * mod.R
+                self._increase_uncertainty(
+                    model=mod, discount_factors=self._discount_intervention)
 
             # Saving posterior state parameters
             dict_state_parms["posterior"]["m"].append(mod.m)
             dict_state_parms["posterior"]["C"].append(mod.C)
+            dict_state_parms["posterior"]["s"].append(mod.s[0][0])
+            dict_state_parms["posterior"]["df"].append(mod.n-1)
 
             # Saving predictive parameters
             dict_predictive["t"].append(t+1)
@@ -530,6 +578,9 @@ class Monitoring:
             dict_predictive["l_lower"].append(lt[1])
             # end loop
 
+        df_predictive = pd.DataFrame(dict_predictive)
+
+        # Organize the results of posterior distribution
         df_posterior = tidy_parameters(
             dict_parameters=dict_state_parms["posterior"],
             entry_m="m", entry_v="C",
@@ -540,43 +591,83 @@ class Monitoring:
         t_index = np.arange(0, len(df_posterior) / n_parms) + 1
         df_posterior["t"] = np.repeat(t_index, n_parms)
         df_posterior["t"] = df_posterior["t"].astype(int)
-        df_posterior = df_posterior[
-            ["t", "parameter", "mean", "variance"]].copy()
+        df_posterior = df_posterior.merge(
+            df_predictive[['t', 'df']], on="t", how="left",
+            validate="many_to_one")
 
-        df_predictive = pd.DataFrame(dict_predictive)
+        df_posterior = df_posterior[
+            ["t", "parameter", "mean", "variance", "df"]].copy()
+        df_posterior = df_posterior.sort_values(
+            ["parameter", "t"]).reset_index(drop=True)
+
+        # Compute credible intervals
         if self._interval:
             df_predictive["ci_lower"] = stats.t.ppf(
-                q=self._level/2, df=df_predictive["df"].values,
+                q=self._level/2,
+                df=df_predictive["df"].values,
                 loc=df_predictive["f"].values,
                 scale=np.sqrt(df_predictive["q"].values))
             df_predictive["ci_upper"] = stats.t.ppf(
-                q=1-self._level/2, df=df_predictive["df"].values,
+                q=1-self._level/2,
+                df=df_predictive["df"].values,
                 loc=df_predictive["f"].values,
                 scale=np.sqrt(df_predictive["q"].values))
 
             df_posterior["ci_lower"] = stats.t.ppf(
-                q=self._level/2, df=df_posterior["t"].values + 1,
+                q=self._level/2,
+                df=df_posterior["df"].values,
                 loc=df_posterior["mean"].values,
                 scale=np.sqrt(df_posterior["variance"].values))
             df_posterior["ci_upper"] = stats.t.ppf(
                 q=1-self._level/2,
-                df=df_posterior["t"].values + 1,
+                df=df_posterior["df"].values,
                 loc=df_posterior["mean"].values,
                 scale=np.sqrt(df_posterior["variance"].values))
 
+        # Output
         out = {"predictive": df_predictive, "posterior": df_posterior}
+
+        # Compute smooth distribution
         if self._smooth:
             smooth_class = Smoothing(
                 mod=self._mod, interval=self._interval, level=self._level)
             dict_smooth = smooth_class.fit(
-                y=pd_y, dict_state_parms=dict_state_parms)
+                y=pd_y, X=pd_X, dict_state_parms=dict_state_parms)
             out = {"filter": out, "smooth": dict_smooth}
 
+        mod.deltrend = self._mod.deltrend
+        mod.delregn = self._mod.delregn
+        mod.delseas = self._mod.delseas
         out["model"] = mod
 
         return out
 
-    def _bayes_factor(self, type, distr):
+    def _increase_uncertainty(self, model: pybats.dglm.dlm,
+                              discount_factors: dict) -> None:
+        """Increase the uncertainty of R matrix using discount factor.
+
+        Parameters
+        ----------
+        model : pybats.dglm.dlm
+            An object of class `pybats.dglm.dlm` with the defined DLM.
+        discount_factors : dict
+            Dictionary with discount factors values for `trend`, `seasonal`,
+            and `reg` block.
+
+        Returns
+        -------
+        None
+            This function just change the model.R and model.W matrices.
+        """
+        model.deltrend = discount_factors["trend"]
+        model.delseas = discount_factors["seasonal"]
+        model.delregn = discount_factors["reg"]
+        mat_discount = model.build_discount_matrix()
+        # Discount information for prior variance at time  t + 1
+        model.W = model.R / mat_discount - model.R
+        model.R = model.R + model.W
+
+    def _bayes_factor(self, distr_fam, distr_type):
         switcher = {
             "normal":
                 {"location": self._bf_normal_location,
@@ -585,7 +676,7 @@ class Monitoring:
                 {"location": self._bf_tstudent_location,
                  "scale": self._bf_tstudent_scale}
         }
-        return switcher[distr][type]
+        return switcher[distr_fam][distr_type]
 
     def _bf_normal_location(self, e, delta, df):
         return np.exp(0.5 * (delta ** 2) - e * delta)
